/***********************************************************
	mwpp - Converts .abcesam files to .window files

	1.00 85-11-28 ldab/hn: Works (hopefully...)
	by Henrik Norbeck, Luxor Datorer AB 

	read.c Contains menu reading functions

	getmenu  - reads a menu
	getcolon - gets an input line from the structure file
		   and splits it at the colons.
	split	 - splits a line into substrings separated 
		   by a certain separator character
	getline  - reads a line from a file
	noescape - deletes 'escape' characters in a string

***********************************************************/

/* Header files */
#include <stdio.h>
#include "mwpp.h"

/*	string arrays for the menus */
extern char nbuf[13][20];
extern char mbuf[13][64];
extern char pbuf[13][64];

extern char *str[5];			/* operative pointers */
extern char buf[256];			/* file string buffer */
extern short eof;

char *astr[5];
char abuf[256];

/*** Getmenu() reads the next menu in the structure file */
getmenu(fp)
FILE *fp;
{
    char *noescape();
    FILE *fopen(), *catf;
    register int i, n, menu, prog;

    /* phase 1: load menu structure from structure file */

    n = menu = prog = 0;
    while (!eof) {
	if (*str[STR_PROG] != '\0') /* programs */
	    strncpy(pbuf[n], str[STR_PROG], 64);
	else {
	    prog--;
	    *pbuf[n] = '\001';
	    strcpy(pbuf[n]+1, str[STR_SYMB]);
	    }
	if (*str[STR_TEXT] != '\0') /* commands */
	    strncpy(mbuf[n], str[STR_TEXT], 64);
	else {
	    menu--;
	    *mbuf[n] = '\001';
	    strcpy(mbuf[n]+1, str[STR_SYMB]);
	    }
	strncpy(nbuf[n], noescape(str[STR_NAME]), 20);
	if (*str[STR_NAME] == '$' && n != 0)
	    break;
	n++;
	eof = (getcolon(fp, buf, str) == NULL);
	}
    if (*nbuf[0] != '$')
	panic("syntax error in structure file");

    /* phase 2: try to solve open links */

    if (menu || prog) {
	if ((catf = fopen(CATALOGUE, "r")) == NULL)
	    panic("no global catalogue");
	while ((menu < 0 || prog < 0) && getcolon(catf, abuf, astr) != NULL) {
	    for (i = n;--i >= 0;)
		if (*pbuf[i] == '\001'
			&& strcmp(astr[CAT_SYMB], pbuf[i] + 1) == 0) {
		    prog++;
		    strncpy(pbuf[i], astr[CAT_PROG], 64 );
		    break;
		    }
	    for (i = n;--i >= 0;)
		if (*mbuf[i] == '\001'
			&& strcmp(astr[CAT_SYMB], mbuf[i] + 1) == 0) {
		    menu++;
		    strncpy(mbuf[i], astr[CAT_TEXT], 64);
		    break;
		    }
	    }
	fclose(catf);
	}

    /* phase 3: mark all unsolved entries */

    if (prog || menu)
	for (i = n; --i >= 0;) {
	    if (*pbuf[i] == '\001')
		strncpy(pbuf[i], "", 64);
	    if (*mbuf[i] == '\001')
		strncpy(mbuf[i], "---", 64);
	    }
    return(n);
}

/***	get input lines

	get a line from fp and split it into a number of
 	substrings, separated by colon. the function returns
	a pointer to an array of character pointers, or NULL
	if failure. the function does not handle overflow.
*/
getcolon(fp, buf, str) 
FILE *fp;
char *buf, *str[];
{
    char *getline();

    if (getline(fp, buf) == NULL)
	return NULL;
    split(5, buf, ':', str);
    return NOTNULL;
}

/*** Getline() reads one line from a file, skipping comment lines */
char *getline(fp, buffer)
FILE *fp;
char *buffer;
{
    register char *p;
    register char c;

    while ((c=getc(fp)) == '#' || c == '\n')	/* Skip comment lines */
	if (c == '#')				/* and empty lines */
	    while ((c=getc(fp)) != EOF && c != '\n');

    p = buffer;
    while (c != EOF) {
	if (c == '\n')
	    if (*(p-1) != '\\')
		break;
	    else
		p--;	/* 'Escaped' return! Skip it! */
	else
	    *p++ = c;
	c = getc(fp);
	}
    if (c == EOF) return (NULL);
    *p = '\0';
    return (buffer);
}

/***	split input string

	Splits the input string into a number of substrings
 	separated by a given char. The function returns a
	pointer to an array of character pointers. No over-
	flow test.

	To type a separator character in a string, an 'escape'
	character is used as a prefix. That character are
	ignored, and any character following is passed by.
	Escape chars are not deleted from the split strings.

	Note: The split is made by placing null separators
	in the input string, thus destroying the original
	contents.
*/
split(cnt, buf, chr, str)
char *buf, **str;
register short cnt;
char chr;
{
    register short int n = 0;
    register char *p=buf, *c=buf;

    while (*p && n < cnt) {
	if (*p != '\\') {		/* ignore escape */
	    if (*p == chr) {		/* break on character */
		str[n++] = c;		/* save the pointer */
		*p = '\0';		/* terminate string */
		c = p + 1;		/* set pointer */
		}
	    }
	else
	    p++;
	p++;				/* next character */
	}
    str[n++] = c;
    while (n < cnt)
	str[n++] = p;	/* fix nulls */
}

/***	delete 'escape' characters */
char *noescape(ptr)
register char *ptr;
{
    register char *p = ptr, *s = ptr;
    do {
	if (*p == '\\') p++;
    } while (*ptr++ = *p++);
    return (s);
}
